/*
 * htmlcallbacks.c
 * htmllite
 *
 * by Doug Wolfe
 * doug@wolfescience.com 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.,  59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
 *
 */

#include <gtk/gtk.h>

#include "xhtmlcallbacks.h"
#include "xhtmlinterface.h"
#include "xhtmlbuffer.h"

#include "support.h"

#include <string.h>

#include "base64.h"


static GdkPixbuf * htmllite_file_encode_pixbuf_base64(const gchar * filename)
{
	GError * error = NULL;
	//Open pixbuf
	GdkPixbuf * mypixbuf = gdk_pixbuf_new_from_file (filename, &error);
	if (error != NULL)
	{
		g_warning ("gdk_pixbuf_new_from_file, %s", error->message);
		return;
	}

	//read file
	gchar* pImageData;
	size_t ImageSize;
	gboolean file_completly_read = g_file_get_contents(filename, &pImageData, &ImageSize, &error);
	if (error != NULL) {
		g_warning ("g_file_get_contents, %s", error->message);
		return ;
	}
	printf("Image Size:%d\n", ImageSize);
	//encode pixbuf
	if(file_completly_read)
	{
		/*if there is trouble with encoding the file
		try increasing the size of BASE64_RESULT_SZ
		base64.c*/
		gchar * encoded;
		encoded = base64_encode_bin(pImageData, ImageSize);
		//printf("Encoded Size:%d\n", strlen(encoded));
		//printf("filename:%s; imagesize:%d; file_encoded:%s; file finished.\n", filename, ImageSize, encoded);

		//set file_string attached to pixbuf
		g_object_set_data(G_OBJECT(mypixbuf), "file_string", encoded);
	}

	//free data
	g_free(pImageData);
	return mypixbuf;
}

static gint cmp_families (const void *a, const void *b)
{
	const char *a_name = pango_font_family_get_name (*(PangoFontFamily **)a);
	const char *b_name = pango_font_family_get_name (*(PangoFontFamily **)b);

	return g_utf8_collate (a_name, b_name);
}

static void htmllite_setup_combobox_font (GtkTextView * textview)
{
	//Get default fonts
	PangoContext * textview_context = gtk_widget_get_pango_context(GTK_WIDGET(textview));
	PangoFontFamily **families;
	int len, n, default_fontfamily_listn;
	pango_context_list_families (textview_context, &families, &len);
	qsort (families, len, sizeof (PangoFontFamily *), cmp_families);
	PangoFontDescription * default_font_desc = gtk_widget_get_style(GTK_WIDGET(textview))->font_desc;
	const gchar * default_fontfamily =  pango_font_description_get_family (default_font_desc);
	gint default_fontsize =  pango_font_description_get_size (default_font_desc);

	//setup combobox_font with all the font families
	GtkWidget * combobox;
	GtkListStore * store;
	GtkTreeIter iter;
	combobox = lookup_widget(GTK_WIDGET(textview), "combobox_font");
	store = gtk_list_store_new (1, G_TYPE_STRING);
	gtk_combo_box_set_model (GTK_COMBO_BOX(combobox), GTK_TREE_MODEL(store));
	for (n = 0;  n < len;  n++)
	{
		const gchar * fontname =  pango_font_family_get_name (families[n]);
		//gtk_combo_box_append_text(GTK_COMBO_BOX(combobox_font), fontname);
		gtk_list_store_append(store, &iter);
		gtk_list_store_set (store, &iter, 0, fontname, -1);
		if(g_ascii_strcasecmp(default_fontfamily, fontname)==0)
		{
			default_fontfamily_listn = n;
		}
	}
	gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), default_fontfamily_listn);
	g_free (families);
	
	//setup combobox_fontsize_absolute
	combobox = lookup_widget(GTK_WIDGET(textview), "combobox_fontsize_absolute");
	store = gtk_list_store_new (1, G_TYPE_STRING);
	gtk_combo_box_set_model (GTK_COMBO_BOX(combobox), GTK_TREE_MODEL(store));
	gchar *sizes[8] = {"Auto-Size", "xx-small", "x-small", "small", "medium", "large", "x-large", "xx-large"};
	for (n = 0;  n < 8;  n++)
	{
		gtk_list_store_append(store, &iter);
		gtk_list_store_set (store, &iter, 0, g_strdup(sizes[n]), -1);
	}
	gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), 0);
}

GtkTextBuffer * buffer;
GtkTextView * textview;
GtkTextBuffer * buffer_output;

void on_dialoglite_show  (GtkWidget * widget, gpointer user_data)
{
}

void setup_buffer (GtkWidget * widget, gchar * text, gboolean * debug)
{
     //create textview buffer
	textview = GTK_TEXT_VIEW(lookup_widget(widget, "textview1"));
	buffer = create_buffer (textview, TRUE);
	
	//create textview_Output buffer
	GtkWidget * textview_Output = lookup_widget(widget, "textview_Output");
	GtkTextTagTable * tagtable_output =  gtk_text_tag_table_new();
	buffer_output = gtk_text_buffer_new (tagtable_output);
	gtk_text_view_set_buffer(GTK_TEXT_VIEW(textview_Output) , buffer_output);
	
	//adjust comboboxes
	GtkWidget * combobox_fontsize_absolute = lookup_widget(widget, "combobox_fontsize_absolute");
	gtk_combo_box_set_active(GTK_COMBO_BOX(combobox_fontsize_absolute), 0);
	htmllite_setup_combobox_font (textview);
	
	//toggle display of textview_Output and Debugging buttons (Export/Import XHTML buttons)
    if(!debug)
	{
		GtkWidget * vbox_debug = lookup_widget(GTK_WIDGET(widget), "vbox_debug");
		gtk_widget_hide(vbox_debug);
	}
	set_XHTML(buffer, text);
}
 
void on_toolbtnCut_clicked  (GtkToolButton * toolbutton, gpointer user_data)
{
	gtk_text_buffer_cut_clipboard(buffer, gtk_clipboard_get(GDK_NONE), TRUE);
}

void on_toolbtnCopy_clicked (GtkToolButton * toolbutton, gpointer user_data)
{
	gtk_text_buffer_copy_clipboard(buffer, gtk_clipboard_get(GDK_NONE));
}

void on_toolbtnPaste_clicked (GtkToolButton * toolbutton, gpointer user_data)
{
	gtk_text_buffer_paste_clipboard(buffer, gtk_clipboard_get(GDK_NONE), NULL, TRUE);
}

void on_toolbtnDeleteText_clicked (GtkToolButton * toolbutton, gpointer user_data)
{
	gtk_text_buffer_delete_selection(buffer, TRUE, TRUE);
}

void on_toolbtnClear_clicked (GtkToolButton * toolbutton, gpointer user_data)
{
	GtkTextIter begin, end;
	gtk_text_buffer_get_start_iter (buffer, &begin);
	gtk_text_buffer_get_end_iter (buffer, &end);
	gtk_text_buffer_remove_all_tags(buffer, &begin, &end);
}

void on_toolbtnUndo_clicked (GtkToolButton * toolbutton, gpointer user_data)
{
	Undo(buffer);
}


void on_toolbtnRedo_clicked (GtkToolButton * toolbutton, gpointer user_data)
{
	Redo(buffer);
}


void on_toggletoolbtn_bold_toggled (GtkToggleToolButton *toggletoolbutton, gpointer user_data)
{
	toggle_Bold(buffer, gtk_toggle_tool_button_get_active(toggletoolbutton));
}

void on_toggletoolbtn_italic_toggled (GtkToggleToolButton *toggletoolbutton, gpointer user_data)
{
	toggle_Italic(buffer, gtk_toggle_tool_button_get_active(toggletoolbutton));
}

void on_toggletoolbtn_underline_toggled (GtkToggleToolButton *toggletoolbutton, gpointer user_data)
{
	toggle_Underline(buffer, gtk_toggle_tool_button_get_active(toggletoolbutton));
}

void on_toggletoolbtn_strike_toggled (GtkToggleToolButton *toggletoolbutton, gpointer user_data)
{
	toggle_Strikethrough(buffer, gtk_toggle_tool_button_get_active(toggletoolbutton)); //Strikethrough
}

void on_toggletoolbtn_superscript_toggled (GtkToggleToolButton *toggletoolbutton, gpointer user_data)
{
	toggle_Superscript(buffer, gtk_toggle_tool_button_get_active(toggletoolbutton));
}

void
on_toggletoolbtn_subscript_toggled (GtkToggleToolButton *toggletoolbutton, gpointer  user_data)
{
	toggle_Subscript(buffer, gtk_toggle_tool_button_get_active(toggletoolbutton));
}


void on_toolbtn_color_clicked (GtkToolButton * toolbutton, gpointer user_data)
{
	open_font_color_dialog(buffer);
}

void on_toolbutton_font_backgroundcolor_clicked (GtkToolButton *toolbutton, gpointer user_data)
{
	open_highlight_color_dialog(buffer);
}

void on_toolbuttonSymbol_clicked (GtkToolButton * toolbutton, gpointer  user_data)
{
	open_symbol_dialog(buffer);
}

void on_btn_ExportHtml_clicked (GtkButton * button, gpointer user_data)
{
	GtkTextIter begin, end;
	gtk_text_buffer_get_bounds (buffer, &begin, &end);

	//gchar * xml =  gtk_text_buffer_get_text(buffer, &begin, &end, TRUE);
	gchar * html = WriteXHTML(buffer, "filename");
	gtk_text_buffer_set_text (buffer_output, html, -1);
}

void on_btn_ImportHtml_clicked (GtkButton * button, gpointer user_data)
{
//Get xml from output textview
	GtkTextIter begin, end;
	gtk_text_buffer_get_bounds  (buffer_output, &begin, &end);

	gchar * xml =  gtk_text_buffer_get_text(buffer_output, &begin, &end, TRUE);
	printf("xml:\n%s\n", xml);
	import_XHTML (buffer, xml, FALSE);
}

gchar * html_response = NULL;

void on_dialoglite_response (GtkDialog * dialog, gint response_id, gpointer user_data)
{
	if (response_id == GTK_RESPONSE_OK)
	{
		html_response = WriteXHTML(buffer, "filename");
		printf("return XML:%s", html_response);
	}
	gtk_widget_hide(GTK_WIDGET(dialog));
}

void on_combobox_unicodetype_changed (GtkComboBox * combobox, gpointer user_data)
{
	GtkWidget *tableSymbol = lookup_widget(GTK_WIDGET(combobox), "tableSymbol");
	//rebuild table
	gtk_container_foreach (GTK_CONTAINER (tableSymbol),
			       (GtkCallback) gtk_widget_destroy,
			       NULL);
	
	int n, r, c;
	r=0;
	c=0;

	int start_char = 0;
	int end_char = 0;
	
	int chosen_unicode_type = gtk_combo_box_get_active (GTK_COMBO_BOX(combobox));
	//char * chosen_unicode_text=  gtk_combo_box_get_active_text(GTK_COMBO_BOX(combobox));
	printf("Chosen Unicode Type:%d", chosen_unicode_type);
	switch (chosen_unicode_type)
	{
		case (0): //Arabic  U+0600  -   U+06FF  (1536-1791)
			start_char = 1536;
			end_char = 1791;
			break;
		case (1): //Arrows Unicode U+2190 ? U+21FF   (8592-8703)
			start_char = 8592;
			end_char = 8703;
			break;
//		case (1): //Arrows-A Supplemental U+2190 ? U+21FF   (8592-8703)
//			start_char = 8592;
//			end_char = 8703;
//			break;
//		case (1): //Arrows-B Supplemental U+2190 ? U+21FF   (8592-8703)
//			start_char = 8592;
//			end_char = 8703;
//			break;
		case (2): //Basic Latin Unicode U+0000 ? U+007F   (0-127)
			start_char = 0;
			end_char = 127;
			break;
		case (3): //Currency Symbols Unicode U+20A0 ? U+20CF   (8352-8399)
			start_char = 0;
			end_char = 127;
			break;
		case (4): //General Punctuation Unicode U+2000 ? U+206F   (8192-8303)
			start_char = 8192;
			end_char = 8303;
			break;
		case (5): //Geometric Shapes Unicode U+25A0 ? U+25FF   (9632-9727)
			start_char = 9632;
			end_char = 9727;
			break;
		case (6): //Greek Unicode U+0370 ? U+03FF   (880-1023)
			start_char = 880;
			end_char = 1023;
			break;
		case (7): //Latin-1 Supplement Unicode U+0080 ? U+00FF   (128-255)
			start_char = 128;
			end_char = 255;
			break;
		case (8): //Latin Extended-A Unicode U+0100 ? U+017F   (256-383)
			start_char = 256;
			end_char = 383;
			break;
		case (9): //Latin Extended-B Unicode U+0180 ? U+024F   (384-591)
			start_char = 384;
			end_char = 591;
			break;
		case (10): //Letterlike Symbols Unicode U+2100 ? U+214F   (8448-8527)
			start_char = 8448;
			end_char = 8527;
			break;
		case (11): //Mathematical Operators Unicode U+2200 ? U+22FF   (8704-8959)
			start_char = 8704;
			end_char = 8959;
			break;
		case (12): //Miscellaneous Technical Unicode U+2300 ? U+23FF   (8960-9215)
			start_char = 8960;
			end_char = 9215;
			break;
		case (13): //Miscellaneous Symbols Unicode U+2600 ? U+26FF   (9728-9983)
			start_char = 9728;
			end_char = 9983;
			break;
		case (14): //Spacing Modifier Letters Unicode U+02B0 ? U+02FF   (688-767)
			start_char = 688;
			end_char = 767;
			break;
		default:
			printf("other");
	}
	for(n=start_char; n<end_char; n++)
	{
		static gchar buf[10];
		int len;
		len = g_unichar_to_utf8 (n, buf);
		buf[len] = '\0';

		//printf("n:%d, buf:%s\n", n, buf);
		const gchar *end;
		if(g_utf8_validate (buf,  -1, &end) && !g_unichar_isspace (buf[0]))
		{
			if (strcmp(buf, ""))
			{
				GtkWidget *btnChar;
				btnChar = gtk_button_new_with_label (buf);
				gtk_button_set_relief(GTK_BUTTON(btnChar), GTK_RELIEF_NONE);
				gtk_table_attach_defaults   (GTK_TABLE(tableSymbol),
						btnChar,
						c,
						c+1,
						r,
						r+1);
				g_signal_connect ((gpointer) btnChar, "enter",
						G_CALLBACK (on_button_Symbol_activate),
						NULL);
				g_signal_connect ((gpointer) btnChar, "clicked",
						G_CALLBACK (on_button_Symbol_clicked),
						NULL);
				c += 1;
				if (c > 10)
				{
					c = 0;
					r += 1;
				}
				gtk_widget_show(btnChar);
			}
		}
		else
		{
			//g_print(" invalid\n");
		}
	}
}

GtkWidget * import_Image_dialog;
void on_toolbutton_Image_clicked (GtkToolButton *toolbutton, gpointer user_data)
{
	if(import_Image_dialog == NULL)
	{
		GtkWidget * dialoglite = lookup_widget(GTK_WIDGET(toolbutton), "dialoglite");
	             
		import_Image_dialog = gtk_file_chooser_dialog_new ("Choose Image", GTK_WINDOW(dialoglite),
				GTK_FILE_CHOOSER_ACTION_OPEN,
				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
		GtkFileFilter *filefilter_image = gtk_file_filter_new();
		gtk_file_filter_set_name(filefilter_image, "Image Files (*gif, *.jpg, *.png)");
		gtk_file_filter_add_pattern(filefilter_image, "*.gif");
		gtk_file_filter_add_pattern(filefilter_image, "*.jpg");
		gtk_file_filter_add_pattern(filefilter_image, "*.png");
		gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (import_Image_dialog), filefilter_image);
        
		GtkFileFilter *filefilter_all = gtk_file_filter_new();
		gtk_file_filter_set_name(filefilter_all, "All Files (*.*)");
		gtk_file_filter_add_pattern(filefilter_all, "*");
		gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (import_Image_dialog), filefilter_all);

	
	}
	if (gtk_dialog_run (GTK_DIALOG (import_Image_dialog)) == GTK_RESPONSE_OK)
	{
		printf("ok\n");
		gtk_widget_hide(import_Image_dialog);
		gchar *filename;
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (import_Image_dialog));
		
		GdkPixbuf * mypixbuf = (GdkPixbuf *)htmllite_file_encode_pixbuf_base64(filename);
		//Insert pixbuf
		GtkTextIter beginselect, endselect;
		gtk_text_buffer_get_selection_bounds(buffer, &beginselect, &endselect);
		gtk_text_buffer_insert_pixbuf(buffer, &beginselect, mypixbuf);

		//free data
		g_free (filename);
	}
	gtk_widget_hide(import_Image_dialog);
}

void on_combobox_fontsize_absolute_changed (GtkComboBox * combobox, gpointer user_data)
{
	printf("on_combobox_fontsize_relative_changed\n");
	gchar * active_text = gtk_combo_box_get_active_text(combobox);
	adjust_fontsize_absolute(buffer, active_text);
}

void on_combobox_font_changed (GtkComboBox * combobox, gpointer user_data)
{
	printf("on_combobox_font_changed\n");
	gchar * active_text = gtk_combo_box_get_active_text(combobox);
	htmllite_change_fontfamily(buffer, GTK_WIDGET(combobox), active_text);
}

void
on_textview_move_cursor                (GtkTextView     *textview,
                                        GtkMovementStep  step,
                                        gint             count,
                                        gboolean         extend_selection,
                                        gpointer         user_data)
{
	printf("move-cursor\n");
}


gboolean
on_textview_focus_out_event            (GtkWidget       *widget,
                                        GdkEventFocus   *event,
                                        gpointer         user_data)
{

  return FALSE;
}


void
on_textview_insert_at_cursor           (GtkTextView     *textview,
                                        gchar           *string,
                                        gpointer         user_data)
{
printf("insert-cursor\n");
}

static void xhtml_track_tags(GtkWidget * widget)
{
	//get tags
	GtkTextIter start, end;
	gtk_text_buffer_get_selection_bounds(buffer, &start, &end);
	gtk_text_iter_backward_char(&start);
	GSList * tags =  gtk_text_iter_get_tags(&start);

	//get widgets
	GtkWidget * toggletoolbtn_bold = lookup_widget(widget, "toggletoolbtn_bold");
	GtkWidget * toggletoolbtn_italic = lookup_widget(widget, "toggletoolbtn_italic");
	GtkWidget * toggletoolbtn_underline = lookup_widget(widget, "toggletoolbtn_underline");
	GtkWidget * toggletoolbtn_strike = lookup_widget(widget, "toggletoolbtn_strike");
	GtkWidget * toggletoolbtn_superscript = lookup_widget(widget, "toggletoolbtn_superscript");
	GtkWidget * toggletoolbtn_subscript = lookup_widget(widget, "toggletoolbtn_subscript");
	
	GtkWidget * combobox_font = lookup_widget(widget, "combobox_font");
	GtkWidget * combobox_fontsize_absolute = lookup_widget(widget, "combobox_fontsize_absolute");
	GtkWidget * combobox_fontsize_pt = lookup_widget(widget, "combobox_fontsize_pt");
	//TODO: track color as well: change btn background
	
	//block toggle handlers
	g_signal_handlers_block_matched (GTK_OBJECT(toggletoolbtn_bold), G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
	g_signal_handlers_block_matched (GTK_OBJECT(toggletoolbtn_italic), G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
	g_signal_handlers_block_matched (GTK_OBJECT(toggletoolbtn_underline), G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
	g_signal_handlers_block_matched (GTK_OBJECT(toggletoolbtn_strike), G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
	g_signal_handlers_block_matched (GTK_OBJECT(toggletoolbtn_superscript), G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
	g_signal_handlers_block_matched (GTK_OBJECT(toggletoolbtn_subscript), G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);

	g_signal_handlers_block_matched (GTK_OBJECT(combobox_font), G_SIGNAL_MATCH_FUNC, 0, 0, NULL, G_CALLBACK (on_combobox_font_changed), NULL);
	g_signal_handlers_block_matched (GTK_OBJECT(combobox_fontsize_absolute), G_SIGNAL_MATCH_FUNC, 0, 0, NULL, G_CALLBACK (on_combobox_font_changed), NULL);
	g_signal_handlers_block_matched (GTK_OBJECT(combobox_fontsize_pt), G_SIGNAL_MATCH_FUNC, 0, 0, NULL, G_CALLBACK (on_combobox_font_changed), NULL);

	//set toggle button active to FALSE
	gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(toggletoolbtn_bold), FALSE);
	gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(toggletoolbtn_italic), FALSE);
	gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(toggletoolbtn_underline), FALSE);
	gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(toggletoolbtn_strike), FALSE);
	gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(toggletoolbtn_superscript), FALSE);
	gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(toggletoolbtn_subscript), FALSE);

	//get default font
	PangoFontDescription * default_font_desc = gtk_widget_get_style(GTK_WIDGET(textview))->font_desc;
	const gchar * default_fontfamily =  pango_font_description_get_family (default_font_desc);
	gint default_fontsize =  pango_font_description_get_size (default_font_desc);
	//set default fontfamily
	GtkTreeIter iter;
	gchar * iter_string;
	GtkTreeModel * model_fontfamily = gtk_combo_box_get_model (GTK_COMBO_BOX(combobox_font));
	gboolean has_iter = gtk_tree_model_get_iter_first (model_fontfamily, &iter);
	while(has_iter)
	{
		gtk_tree_model_get(model_fontfamily, &iter, 0, &iter_string, -1);
		if(strcmp(default_fontfamily, iter_string) == 0)
		{
			gtk_combo_box_set_active_iter (GTK_COMBO_BOX(combobox_font), &iter);
			break;
		}
		has_iter = gtk_tree_model_iter_next(model_fontfamily, &iter);
	}
	//set default fontfamily
	gtk_combo_box_set_active (GTK_COMBO_BOX(combobox_fontsize_absolute), 0);


	//Check tags
	int n;
	guint len =  g_slist_length(tags);
	for(n = 0; n < len; n++)
	{
		GtkTextTag * tag = g_slist_nth_data (tags, n);
		if(strcmp(tag->name, "b") == 0)
		{
			gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(toggletoolbtn_bold), TRUE);
		}
		else if(strcmp(tag->name, "i") == 0)
		{
			gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(toggletoolbtn_italic), TRUE);
		}
		else if(strcmp(tag->name, "u") == 0)
		{
			gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(toggletoolbtn_underline), TRUE);
		}
		else if(strcmp(tag->name, "del") == 0)
		{
			gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(toggletoolbtn_strike), TRUE);
		}
		else if(strcmp(tag->name, "sup") == 0)
		{
			gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(toggletoolbtn_superscript), TRUE);
		}
		else if(strcmp(tag->name, "sub") == 0)
		{
			gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(toggletoolbtn_subscript), TRUE);
		}
		else if(g_strrstr(tag->name, "face")!= NULL)
		{
			gchar * fontfamily;
			g_object_get (tag, "family", &fontfamily, NULL);

			//set selected font
			has_iter = gtk_tree_model_get_iter_first (model_fontfamily, &iter);
			while(has_iter)
			{
				gtk_tree_model_get(model_fontfamily, &iter, 0, &iter_string, -1);
				if(strcmp(fontfamily, iter_string) == 0)
				{
					gtk_combo_box_set_active_iter (GTK_COMBO_BOX(combobox_font), &iter);
					break;
				}
				has_iter = gtk_tree_model_iter_next(model_fontfamily, &iter);
			}
		}
		else if(g_strrstr(tag->name, "font-size")!= NULL)
		{
			gchar * fontsize = g_object_get_data(G_OBJECT(tag), "font-size_absolute");
			printf("fontsize:%s\n", fontsize);

			//set selected font
			GtkTreeModel * model_fontsize_absolute = gtk_combo_box_get_model (GTK_COMBO_BOX(combobox_fontsize_absolute));
			has_iter = gtk_tree_model_get_iter_first (model_fontsize_absolute, &iter);
			while(has_iter)
			{
				gtk_tree_model_get(model_fontsize_absolute, &iter, 0, &iter_string, -1);
				if(strcmp(fontsize, iter_string) == 0)
				{
					gtk_combo_box_set_active_iter (GTK_COMBO_BOX(combobox_fontsize_absolute), &iter);
					break;
				}
				has_iter = gtk_tree_model_iter_next(model_fontsize_absolute, &iter);
			}	
		}
	}

	//
	g_signal_handlers_unblock_matched (GTK_OBJECT(toggletoolbtn_bold), G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
	g_signal_handlers_unblock_matched (GTK_OBJECT(toggletoolbtn_italic), G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
	g_signal_handlers_unblock_matched (GTK_OBJECT(toggletoolbtn_underline), G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
	g_signal_handlers_unblock_matched (GTK_OBJECT(toggletoolbtn_strike), G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
	g_signal_handlers_unblock_matched (GTK_OBJECT(toggletoolbtn_superscript), G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
	g_signal_handlers_unblock_matched (GTK_OBJECT(toggletoolbtn_subscript), G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
	
	g_signal_handlers_unblock_matched (GTK_OBJECT(combobox_font), G_SIGNAL_MATCH_FUNC, 0, 0, NULL, G_CALLBACK (on_combobox_font_changed), NULL);
	g_signal_handlers_unblock_matched (GTK_OBJECT(combobox_fontsize_absolute), G_SIGNAL_MATCH_FUNC, 0, 0, NULL, G_CALLBACK (on_combobox_font_changed), NULL);
	g_signal_handlers_unblock_matched (GTK_OBJECT(combobox_fontsize_pt), G_SIGNAL_MATCH_FUNC, 0, 0, NULL, G_CALLBACK (on_combobox_font_changed), NULL);
	
	printf("on_textview_key_release_event\n");
}

gboolean
on_textview_key_release_event          (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	xhtml_track_tags(widget);
	return FALSE;
}


gboolean
on_textview_button_press_event         (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

  return FALSE;
}


gboolean
on_textview_button_release_event       (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	xhtml_track_tags(widget);
	return FALSE;
}
